local utility_module = require "utility"

local builder = {}

local function getPrimaryExternalStoragePath() return "/storage/emulated/0/" end
local function getSecondaryExternalStoragePath() return "/sdcard/" end
local function getBasePath() local pkg=activity.getPackageName(); local path="/data/user/0/"..pkg.."/app_lua/"; local f_s,f_o=pcall(function()return File(path)end); if f_s and f_o and not f_o.exists()then os.execute("mkdir -p \""..path.."\"") elseif not f_s then print("Warning: Could not create File object for base path: "..path)end; return path end
local settingsFile=getBasePath().."settings.txt"; builder.useRoot=false; builder.isManagingAppModules=false
local function shQuote(s) if s==nil then return"''"end; return"'"..s:gsub("'","'\"'\"'").."'"end
local function exec(cmd) local fullCmd=cmd; if builder.useRoot then fullCmd="su -c \""..cmd.."\""end; return os.execute(fullCmd) end

local criticalModules = { ["import.lua"] = true, ["console.lua"] = true }
local function isCriticalModule(filename)
    if not filename then return false end
    local plainFilename = filename:match("([^/]+)$")
    if not plainFilename then plainFilename = filename end
    return criticalModules[plainFilename:lower()]
end

local function showCriticalModuleDialog(action)
    builder.show("Action Denied", "The modules 'import.lua' and 'console.lua' are critical and cannot be "..(action or "modified")..".")
end

local function listDir(path)
  local entries={}; local files={}; local dirs={}
  if not path or path=="" then print("listDir: called with empty path."); return entries end
  if not path:match("/$") then path=path.."/" end
  if builder.useRoot then
    local p = io.popen("su -c \"cd " .. shQuote(path) .. " && ls -Fp\"")
    if p then local output,popen_err=p:read("*a");p:close(); if output then for entry in output:gmatch("[^\n]+")do if entry:match("/$")then table.insert(dirs,entry)elseif entry~=""and not entry:match("[*@|=]$")then table.insert(files,entry)end end elseif popen_err then print("listDir (root) popen error for path: "..path.." Error: "..tostring(popen_err))end
    else print("listDir (root) failed to open pipe for path: "..path)end
  else
    local success_list,raw_or_error=pcall(function() local dirFile_s,dirFile_o=pcall(function()return File(path)end); if not(dirFile_s and dirFile_o)then error("Could not create File object for: "..path)end; if dirFile_o.isDirectory()then return dirFile_o.list()elseif not dirFile_o.exists()then print("listDir (non-root): Path does not exist: "..path);return nil else print("listDir (non-root): Path is not a directory: "..path);return nil end end)
    if success_list and raw_or_error then
      for _,entryName in ipairs(luajava.astable(raw_or_error))do
        if entryName~="."and entryName~=".."then
          local fileItem_s,fileItem_o=pcall(function()return File(path..entryName)end)
          if fileItem_s and fileItem_o then if fileItem_o.isDirectory()then table.insert(dirs,entryName.."/")else table.insert(files,entryName)end
          else print("listDir (non-root): Could not create File object for item: "..path..entryName..". Error: "..tostring(fileItem_o))end
        end
      end
    elseif not success_list then print("listDir (non-root) critical error for path: "..path.." Error: "..tostring(raw_or_error))end
  end
  table.sort(dirs,function(a,b)return a:lower()<b:lower()end);table.sort(files,function(a,b)return a:lower()<b:lower()end)
  for _,de in ipairs(dirs)do table.insert(entries,de)end;for _,fe in ipairs(files)do table.insert(entries,fe)end
  return entries
end

local function listLuaFilesAndDirsForInstaller(path)
  local entries={}; local files={}; local dirs={}
  if not path or path=="" then print("listLuaFilesAndDirsForInstaller: called with empty path."); return entries end
  if not path:match("/$") then path=path.."/" end
  if builder.useRoot then
    local p=io.popen("su -c \"cd "..shQuote(path).." && ls -Fp\""); if p then local o,e=p:read("*a");p:close(); if o then for en in o:gmatch("[^\n]+")do if en:match("/$")then table.insert(dirs,en)elseif (en:lower():match("%.lua$") or en:lower():match("%.zip$")) and en~=""and not en:match("[*@|=]$")then table.insert(files,en)end end elseif e then print("listLuaFilesAndDirsForInstaller (root) popen error: "..path.." Error: "..tostring(e))end else print("listLuaFilesAndDirsForInstaller (root) failed pipe: "..path)end
  else
    local s_l,r_e=pcall(function() local df_s,df_o=pcall(function()return File(path)end); if not(df_s and df_o)then error("No File obj: "..path)end; if df_o.isDirectory()then return df_o.list()end;return nil end)
    if s_l and r_e then for _,eN in ipairs(luajava.astable(r_e))do if eN~="."and eN~=".."then local fi_s,fi_o=pcall(function()return File(path..eN)end); if fi_s and fi_o then if fi_o.isDirectory()then table.insert(dirs,eN.."/")elseif (eN:lower():match("%.lua$") or eN:lower():match("%.zip$")) then table.insert(files,eN)end end end end
    elseif not s_l then print("listLuaFilesAndDirsForInstaller (non-root) error: "..path.." Error: "..tostring(r_e))end
  end
  table.sort(dirs,function(a,b)return a:lower()<b:lower()end);table.sort(files,function(a,b)return a:lower()<b:lower()end)
  for _,de in ipairs(dirs)do table.insert(entries,de)end;for _,fe in ipairs(files)do table.insert(entries,fe)end
  return entries
end

local function loadSettings() local f=io.open(settingsFile,"r"); if f then for l in f:lines()do if l:match("^useRoot=true")then builder.useRoot=true end end; f:close() end end
local function saveSettings() local f=io.open(settingsFile,"w"); if f then f:write("useRoot="..tostring(builder.useRoot).."\n"); f:close() else builder.show("Settings Save Error","Could not write to the settings file.") end; activity.runOnUiThread(function() builder.updateToolboxVisibility() end) end
function builder.show(title,msg) AlertDialog.Builder(activity).setTitle(title).setMessage(tostring(msg)).setPositiveButton("OK",nil).show() end
function builder.writeModule(moduleName, moduleCode, basePathToWrite) local effectiveBasePath = basePathToWrite or getBasePath(); if not effectiveBasePath then print("writeModule: effectiveBasePath is nil!"); return end; if not effectiveBasePath:match("/$")then effectiveBasePath = effectiveBasePath .. "/" end; local filePath = effectiveBasePath .. moduleName:gsub("%.","/") .. ".lua"; local folder = filePath:match("(.*/)"); if folder then exec("mkdir -p "..shQuote(folder)) else builder.show("Module Creation Failed","Could not determine directory for module:\n"..filePath); return end; local f,err = io.open(filePath,"w"); if not f then builder.show("Module Creation Failed","Could not open file for writing:\n"..filePath.."\nDetails: "..tostring(err)); return end; f:write(moduleCode); f:close(); builder.show("Module Created Successfully!","Module '"..moduleName.."' saved to:\n"..filePath.."\n\nYou can now use: require\""..moduleName.."\"") end
function builder.readFileWithRootFallback(filePath) local success_direct, content_or_err_direct = pcall(function() local f = io.open(filePath, "r"); if not f then return nil, "File open failed." end; local data = f:read("*a"); f:close(); return data end); if success_direct and type(content_or_err_direct) == "string" then return content_or_err_direct, nil end; if builder.useRoot then print("Direct read for '"..filePath.."' failed. Attempting root. Error: " .. tostring(content_or_err_direct)); local partition_to_remount_str = nil; local common_system_partitions = {"/system", "/vendor", "/product", "/system_ext", "/odm"}; for _, sys_path_root in ipairs(common_system_partitions) do if filePath:sub(1, #sys_path_root + 1) == sys_path_root .. "/" then partition_to_remount_str = sys_path_root; break end end; if partition_to_remount_str then local cmd_chain = string.format("mount -o rw,remount %s && cat %s && mount -o ro,remount %s", shQuote(partition_to_remount_str), shQuote(filePath), shQuote(partition_to_remount_str)); local pipe = io.popen("su -c \"" .. cmd_chain .. "\""); if pipe then local root_content = pipe:read("*a"); local exit_status = pipe:close(); if exit_status == 0 and root_content ~= nil then return root_content, nil else return nil, "Root read (remount) failed. Exit: " .. tostring(exit_status) end else return nil, "Failed to open pipe for root read (remount)." end else local cmd_simple_cat = string.format("cat %s", shQuote(filePath)); local pipe_simple = io.popen("su -c \"" .. cmd_simple_cat .. "\""); if pipe_simple then local root_content_simple = pipe_simple:read("*a"); local exit_status_simple = pipe_simple:close(); if exit_status_simple == 0 and root_content_simple ~= nil then return root_content_simple, nil else return nil, "Simple root read failed. Exit: " .. tostring(exit_status_simple) end else return nil, "Failed to open pipe for simple root read." end end end; return nil, "Failed to read file.\nDirect access: " ..tostring(content_or_err_direct) end

local function processInstallationWithBackupOption(modulesToInstall)
    local appBasePath = getBasePath()
    local existingModulesToProcess = {}
    local newModulesToInstall = {}

    for _, modInfo in ipairs(modulesToInstall) do
        local appModulePath = appBasePath .. modInfo.name:gsub("%.", "/") .. ".lua"
        local appModuleFile_s, appModuleFile_o = pcall(function() return File(appModulePath) end)
        if appModuleFile_s and appModuleFile_o and appModuleFile_o.exists() and not appModuleFile_o.isDirectory() then
            table.insert(existingModulesToProcess, {
                name = modInfo.name,
                path = appModulePath,
                installData = modInfo
            })
        else
            table.insert(newModulesToInstall, modInfo)
        end
    end

    if #existingModulesToProcess == 0 then
        local progress = ProgressDialog(activity)
        progress.setMessage("Installing " .. #newModulesToInstall .. " module(s)..."); progress.setCancelable(false); progress.show()
        activity.runOnUiThread(function()
          for _, modInfo in ipairs(newModulesToInstall) do
              builder.writeModule(modInfo.name, modInfo.content, appBasePath)
          end
          progress.dismiss()
          if #newModulesToInstall > 0 then
             builder.show("Installation Summary", #newModulesToInstall .. " new module(s) installed.")
          else
             builder.show("Installation Info", "No new modules were installed.")
          end
        end)
        return
    end

    local existingModuleNames = {}
    for _, modInfo in ipairs(existingModulesToProcess) do table.insert(existingModuleNames, modInfo.name) end

    local msg = "The following " .. #existingModulesToProcess .. " module(s) already exist:\n" .. table.concat(existingModuleNames, "\n")
    if #newModulesToInstall > 0 then
        msg = msg .. "\n\nAdditionally, " .. #newModulesToInstall .. " new module(s) will be installed."
    else
        msg = msg .. "\n\nNo other new modules found in selection to install."
    end

    AlertDialog.Builder(activity)
        .setTitle("Existing Modules Found")
        .setMessage(msg)
        .setPositiveButton("Backup & Overwrite All", DialogInterface.OnClickListener{
            onClick = function()
                local backupPathBase = getPrimaryExternalStoragePath() .. "ModuleBackups/" .. os.date("%Y-%m-%d_%H-%M-%S") .. "/"
                exec("mkdir -p " .. shQuote(backupPathBase))
                local progress = ProgressDialog(activity)
                progress.setMessage("Backing up and installing..."); progress.setCancelable(false)
                activity.runOnUiThread(function() progress.show() end)

                local backedUpCount = 0
                local failedBackupCount = 0

                activity.runOnUiThread(function()
                  for _, modInfo in ipairs(existingModulesToProcess) do
                      local existingContent, err = builder.readFileWithRootFallback(modInfo.path)
                      if existingContent then
                          local backupModuleDir = backupPathBase .. modInfo.name:gsub("%.", "/")
                          backupModuleDir = backupModuleDir:match("(.*/)")
                          if backupModuleDir then exec("mkdir -p "..shQuote(backupModuleDir)) end

                          local backupModuleFile = backupPathBase .. modInfo.name:gsub("%.", "/") .. ".lua"
                          local bf = io.open(backupModuleFile, "w")
                          if bf then
                              bf:write(existingContent); bf:close()
                              backedUpCount = backedUpCount + 1
                          else
                              print("Failed to write backup for " .. modInfo.name .. " to " .. backupModuleFile)
                              failedBackupCount = failedBackupCount + 1
                          end
                      else
                           print("Failed to read existing module for backup: " .. modInfo.name .. " Error: " .. tostring(err))
                           failedBackupCount = failedBackupCount + 1
                      end
                      builder.writeModule(modInfo.installData.name, modInfo.installData.content, appBasePath)
                  end
                  for _, modInfo in ipairs(newModulesToInstall) do
                      builder.writeModule(modInfo.name, modInfo.content, appBasePath)
                  end
                  progress.dismiss()
                  local summaryMsg = (#existingModulesToProcess + #newModulesToInstall) .. " module(s) processed.\n" ..
                                     backedUpCount .. " module(s) backed up to " .. backupPathBase .. "\n" ..
                                     failedBackupCount .. " module(s) failed to backup."
                  builder.show("Installation Complete", summaryMsg)
                end)
            end
        })
        .setNeutralButton("Overwrite All (No Backup)", DialogInterface.OnClickListener{
            onClick = function()
                local progress = ProgressDialog(activity)
                progress.setMessage("Overwriting and installing..."); progress.setCancelable(false)
                activity.runOnUiThread(function() progress.show() end)

                activity.runOnUiThread(function()
                  for _, modInfo in ipairs(existingModulesToProcess) do
                      builder.writeModule(modInfo.installData.name, modInfo.installData.content, appBasePath)
                  end
                  for _, modInfo in ipairs(newModulesToInstall) do
                      builder.writeModule(modInfo.name, modInfo.content, appBasePath)
                  end
                  progress.dismiss()
                  builder.show("Installation Complete", (#existingModulesToProcess + #newModulesToInstall) .. " module(s) installed/overwritten.")
                end)
            end
        })
        .setNegativeButton("Cancel Installation", nil)
        .show()
end

local function handleZipFileInstallation(zipFilePath)
    local luaModulesInZip = {}
    local zipFile_instance
    local entries_enum
    local inputStream

    local zip_pcall_success, zip_result_or_error
    zip_pcall_success, zip_result_or_error = pcall(function()
        return ZipFile(zipFilePath) 
    end)

    if zip_pcall_success then
        zipFile_instance = zip_result_or_error
        if not zipFile_instance then 
            builder.show("Zip Error", "Could not open zip file: " .. zipFilePath .. "\nThe global 'ZipFile' was called but returned no object (nil). Ensure 'ZipFile' is correctly provided by the environment.")
            return
        end
    else
        builder.show("Zip Error", "Could not open zip file: " .. zipFilePath ..
                                  "\nEnsure the global 'ZipFile' is available and the file path is valid." ..
                                  "\nDetails: " .. tostring(zip_result_or_error))
        return
    end
    
    if not _G.Byte or not _G.Byte.TYPE then 
        builder.show("Zip Environment Error", "Cannot process Zip contents. The global 'Byte' class or its 'TYPE' field is not available. Ensure 'Byte' (java.lang.Byte) is correctly provided by the environment.")
        pcall(function() if zipFile_instance and zipFile_instance.close then zipFile_instance.close() end end)
        return
    end
    
    local entries_pcall_success, entries_result_or_error
    entries_pcall_success, entries_result_or_error = pcall(function()
        return zipFile_instance.entries() 
    end)

    if entries_pcall_success then
        entries_enum = entries_result_or_error
        if not entries_enum then
            builder.show("Zip Error", "Could not get entry enumerator from ZipFile. The 'entries()' method returned nil.")
            pcall(function() if zipFile_instance and zipFile_instance.close then zipFile_instance.close() end end)
            return
        end
    else
        builder.show("Zip Error", "Failed to call 'entries()' method on ZipFile object.\nDetails: " .. tostring(entries_result_or_error))
        pcall(function() if zipFile_instance and zipFile_instance.close then zipFile_instance.close() end end)
        return
    end

    while entries_enum.hasMoreElements() do
        local entry = entries_enum.nextElement()
        local entryName = entry.getName()
        if not entry.isDirectory() and entryName:lower():match("%.lua$") then
            local moduleFileName = entryName:match("([^/]+)$")
            if moduleFileName and moduleFileName:lower() == "main.lua" then
                print("Skipping main.lua found in zip: " .. entryName)
                goto continue_zip_entry 
            end

            if moduleFileName then 
                local moduleNameFromZip = moduleFileName:gsub("%.lua$", "")
                
                local get_stream_success, stream_or_error
                get_stream_success, stream_or_error = pcall(function()
                    return zipFile_instance.getInputStream(entry) 
                end)

                if not get_stream_success then
                    print("Error getting InputStream for entry " .. entryName .. ": " .. tostring(stream_or_error))
                    builder.show("Zip Entry Error", "Could not get input stream for: " .. entryName .. "\nDetails: " .. tostring(stream_or_error))
                    goto continue_zip_entry
                end
                inputStream = stream_or_error
                if not inputStream then
                     print("getInputStream for " .. entryName .. " returned nil.")
                     builder.show("Zip Entry Error", "Input stream for " .. entryName .. " was nil.")
                     goto continue_zip_entry
                end

                local buffer = {}
                local bytes_array
                local create_array_success, create_array_error = pcall(function()
                    bytes_array = luajava.newArray(_G.Byte.TYPE, 4096)
                end)

                if not create_array_success then
                    print("Error creating byte array for zip entry " .. entryName .. ": " .. tostring(create_array_error))
                    builder.show("Zip Processing Error", "Failed to create byte array for reading " .. entryName .. "\nDetails: " .. tostring(create_array_error))
                    pcall(function() if inputStream and inputStream.close then inputStream.close() end end)
                    goto continue_zip_entry
                end
                
                local bytesRead
                local read_success, read_error_or_data = pcall(function()
                    while true do
                        bytesRead = inputStream.read(bytes_array) 
                        if bytesRead == -1 then break end 
                        
                        if bytesRead > 0 then 
                            for i = 0, bytesRead - 1 do
                                local java_byte_val = bytes_array[i]
                                local lua_unsigned_byte_val = java_byte_val
                                if lua_unsigned_byte_val < 0 then
                                    lua_unsigned_byte_val = lua_unsigned_byte_val + 256
                                end

                                local char_success, char_val_or_err = pcall(string.char, lua_unsigned_byte_val)
                                if char_success then
                                    table.insert(buffer, char_val_or_err)
                                else
                                    print(string.format("Warning: string.char() failed for original byte %d (unsigned %d) in entry %s. Error: %s", java_byte_val, lua_unsigned_byte_val, entryName, tostring(char_val_or_err)))
                                    -- As per user request, not inserting a placeholder, but logging the issue.
                                end
                            end
                        end
                    end
                end)

                pcall(function() if inputStream and inputStream.close then inputStream.close() end end)

                if read_success then
                    local content = table.concat(buffer)
                    table.insert(luaModulesInZip, {fileName = moduleFileName, name = moduleNameFromZip, content = content, originalPathInZip = entryName})
                else
                    print("Error reading stream for entry " .. entryName .. " in zip: " .. tostring(read_error_or_data))
                    builder.show("Zip Entry Read Error", "Failed to read content of " .. entryName .. "\nDetails: " .. tostring(read_error_or_data))
                end
            end
        end
        ::continue_zip_entry::
    end
    pcall(function() if zipFile_instance and zipFile_instance.close then zipFile_instance.close() end end)

    if #luaModulesInZip == 0 then
        builder.show("Zip Contents", "No valid Lua modules found in this zip file (excluding 'main.lua' or files with read/processing errors).")
        return
    end

    local moduleNamesDisplay = {}
    for _, modInfo in ipairs(luaModulesInZip) do table.insert(moduleNamesDisplay, modInfo.name) end

    local title, message
    if #luaModulesInZip == 1 then
        title = "Install Module from Zip"
        message = "The zip file contains 1 module:\n'" .. luaModulesInZip[1].name .. "'\n\nDo you want to install it?"
    else
        title = "Install Modules from Zip"
        message = "The zip file contains " .. #luaModulesInZip .. " modules:\n" .. table.concat(moduleNamesDisplay, "\n") .. "\n\nDo you want to install them?"
    end

    AlertDialog.Builder(activity)
        .setTitle(title)
        .setMessage(message)
        .setPositiveButton("Install", DialogInterface.OnClickListener{
            onClick = function()
                processInstallationWithBackupOption(luaModulesInZip)
            end
        })
        .setNegativeButton("Cancel", nil)
        .show()
end


function builder.installModuleFromStorage()
    local currentInstallPath = getPrimaryExternalStoragePath()
    local function showModuleFileChooser(path)
        if not path:match("/$") then path = path .. "/" end
        local dirContents = listLuaFilesAndDirsForInstaller(path)
        local displayItems = {}; local actionItems = {}
        local primaryRoot = getPrimaryExternalStoragePath()
        if path ~= "/" and #path > #primaryRoot and path ~= primaryRoot then table.insert(displayItems, "../ (Go Up)"); table.insert(actionItems, {type="goup"}) end
        for _, itemFromListDir in ipairs(dirContents) do table.insert(displayItems, itemFromListDir); table.insert(actionItems, itemFromListDir) end

        local dialogBuilder = AlertDialog.Builder(activity); dialogBuilder.setTitle("Install Module From: " .. path)
        if #displayItems == 0 then dialogBuilder.setMessage("No .lua or .zip files or subdirectories found in this folder.")
        else
            local p_setitems_success, p_setitems_err = pcall(function()
                dialogBuilder.setItems(displayItems, DialogInterface.OnClickListener{
                    onClick = function(_,which)
                        local selectedDisplayItem = displayItems[which+1]; local actualItemForAction = actionItems[which+1]
                        if type(actualItemForAction) == "table" and actualItemForAction.type == "goup" then
                            local parentPath = path:match("(.*/)[^/]+/$"); showModuleFileChooser(parentPath or primaryRoot)
                        elseif type(actualItemForAction) == "string" and actualItemForAction:match("/$") then
                            showModuleFileChooser(path .. actualItemForAction)
                        elseif type(actualItemForAction) == "string" then 
                            local fileName = actualItemForAction
                            if fileName:lower():match("%.zip$") then
                                handleZipFileInstallation(path .. fileName)
                                return
                            elseif fileName:lower():match("%.lua$") then
                                local moduleName = fileName:gsub("%.lua$", "")

                                if fileName:lower() == "main.lua" then
                                    builder.show("Installation Error", "The file 'main.lua' cannot be installed as a standard module. Please choose a different .lua file.")
                                    return
                                end
                                if isCriticalModule(fileName) then
                                    showCriticalModuleDialog("install (overwrite)")
                                    return
                                end

                                local appBasePath = getBasePath()
                                local appModulePath = appBasePath .. moduleName:gsub("%.", "/") .. ".lua"
                                local appModuleFile_s, appModuleFile_o = pcall(function() return File(appModulePath) end)
                                local moduleExistsInAppBase = appModuleFile_s and appModuleFile_o and appModuleFile_o.exists() and not appModuleFile_o.isDirectory()

                                if moduleExistsInAppBase then
                                    AlertDialog.Builder(activity)
                                        .setTitle("Module Exists: " .. moduleName)
                                        .setMessage("The module '" .. moduleName .. "' already exists in the app's internal storage:\n(" .. appModulePath .. ")\n\nWhat would you like to do?")
                                        .setPositiveButton("Update", DialogInterface.OnClickListener{
                                            onClick = function()
                                                if isCriticalModule(moduleName .. ".lua") then
                                                    showCriticalModuleDialog("update")
                                                    return
                                                end
                                                local progress = ProgressDialog(activity); progress.setMessage("Updating '"..moduleName.."' module..."); progress.setCancelable(false); progress.show()
                                                local fileContent, readErr = builder.readFileWithRootFallback(path .. fileName)
                                                activity.runOnUiThread(function() progress.dismiss() end)
                                                if fileContent then
                                                    builder.writeModule(moduleName, fileContent, appBasePath)
                                                else
                                                    builder.show("File Read Error", "Could not read selected file for update: " .. fileName .. "\n\nDetails: " .. tostring(readErr))
                                                end
                                            end
                                        })
                                        .setNeutralButton("Uninstall", DialogInterface.OnClickListener{
                                            onClick = function()
                                                if isCriticalModule(moduleName .. ".lua") then
                                                    showCriticalModuleDialog("uninstall")
                                                    return
                                                end
                                                AlertDialog.Builder(activity)
                                                    .setTitle("Confirm Uninstall")
                                                    .setMessage("Are you sure you want to uninstall the existing module '" .. moduleName .. "' from the app's internal storage?\n(" .. appModulePath .. ")")
                                                    .setPositiveButton("Yes, Uninstall", DialogInterface.OnClickListener{
                                                        onClick = function()
                                                            local fileToDelete = File(appModulePath)
                                                            local baseAppDirFile = File(appBasePath)
                                                            local cmd_rm_file = "rm -f " .. shQuote(appModulePath)
                                                            local rm_file_status = exec(cmd_rm_file)

                                                            if rm_file_status == true then 
                                                                builder.show("Uninstall Successful", "Module '" .. moduleName .. "' has been uninstalled.")
                                                                local parentDirFile = fileToDelete:getParentFile()
                                                                while parentDirFile ~= nil and parentDirFile.exists() and parentDirFile.isDirectory() and
                                                                      not parentDirFile:equals(baseAppDirFile) and
                                                                      parentDirFile:getAbsolutePath():startsWith(baseAppDirFile:getAbsolutePath()) do

                                                                    local filesInDir_j = parentDirFile.list()
                                                                    local filesInDir_l = (filesInDir_j ~= nil) and luajava.astable(filesInDir_j) or {}

                                                                    if #filesInDir_l == 0 then
                                                                        print("Attempting to remove empty directory: " .. parentDirFile:getAbsolutePath())
                                                                        local cmd_rmdir = "rmdir " .. shQuote(parentDirFile:getAbsolutePath())
                                                                        local rmdir_status = exec(cmd_rmdir)
                                                                        if rmdir_status == true then 
                                                                            local nextParent = parentDirFile:getParentFile()
                                                                            if nextParent == nil or not nextParent.exists() then
                                                                                print("Parent directory no longer exists or is null, stopping cleanup.")
                                                                                break
                                                                            end
                                                                            parentDirFile = nextParent
                                                                        else
                                                                            print("Failed to remove directory (or it was not empty): " .. parentDirFile:getAbsolutePath() .. ". rmdir status: " .. tostring(rmdir_status) .. ". Stopping cleanup for this branch.")
                                                                            break
                                                                        end
                                                                    else
                                                                        print("Directory " .. parentDirFile:getAbsolutePath() .. " is not empty. Stopping cleanup for this branch.")
                                                                        break
                                                                    end
                                                                end
                                                            else
                                                                builder.show("Uninstall Failed", "Could not uninstall module '" .. moduleName .. "'.\nThe 'rm -f' command failed or returned a non-zero exit status.\nResult from exec: " .. tostring(rm_file_status))
                                                                print("Uninstall debug: Command '" .. cmd_rm_file .. "' failed. exec result: " .. tostring(rm_file_status))
                                                            end
                                                        end
                                                    })
                                                    .setNegativeButton("Cancel", nil)
                                                    .show()
                                            end
                                        })
                                        .setNegativeButton("Cancel", nil)
                                        .show()
                                else
                                    AlertDialog.Builder(activity)
                                        .setTitle("Install New Module: " .. moduleName)
                                        .setMessage("The module '" .. moduleName .. "' is not found in the app's internal storage.\nSelected file: (" .. path .. fileName .. ")\n\nHow would you like to install it?")
                                        .setPositiveButton("Direct Install (to App)", DialogInterface.OnClickListener{
                                            onClick = function()
                                                local progress = ProgressDialog(activity); progress.setMessage("Installing '"..moduleName.."' module..."); progress.setCancelable(false); progress.show()
                                                local fileContent, readErr = builder.readFileWithRootFallback(path .. fileName)
                                                activity.runOnUiThread(function() progress.dismiss() end)
                                                if fileContent then
                                                    builder.writeModule(moduleName, fileContent, appBasePath)
                                                else
                                                    builder.show("File Read Error", "Could not read the selected file for installation: " .. fileName .. "\n\nDetails: " .. tostring(readErr))
                                                end
                                            end
                                        })
                                        .setNeutralButton("Install to Project (External)", DialogInterface.OnClickListener{
                                            onClick = function()
                                                local progress = ProgressDialog(activity); progress.setMessage("Preparing '"..moduleName.."' for project installation..."); progress.setCancelable(false); progress.show()
                                                local fileContent, readErr = builder.readFileWithRootFallback(path .. fileName)
                                                activity.runOnUiThread(function() progress.dismiss() end)
                                                if fileContent then
                                                    builder.writeModuleToProject(moduleName, fileContent)
                                                else
                                                    builder.show("File Read Error", "Could not read the selected file for project installation: " .. fileName .. "\n\nDetails: " .. tostring(readErr))
                                                end
                                            end
                                        })
                                        .setNegativeButton("Cancel", nil)
                                        .show()
                                end
                            end 
                        end 
                    end 
                }) 
            end) 
            if not p_setitems_success then builder.show("Dialog Display Error", "There was an issue preparing the file list for display.\nDetails: " .. tostring(p_setitems_err)) end
        end
        dialogBuilder.setNegativeButton("Cancel", nil); dialogBuilder.show()
    end
    showModuleFileChooser(currentInstallPath)
end

function builder.writeModuleToProject(moduleName,moduleCode) local target_path=getPrimaryExternalStoragePath(); local function showDirPicker(currentPath) if not currentPath or not currentPath:match("/$") then currentPath=target_path end; if not currentPath:match("/$") then currentPath=currentPath.."/" end; local dirContents=listDir(currentPath); if #dirContents==0 and currentPath==target_path then local secondary_path=getSecondaryExternalStoragePath(); if secondary_path~=target_path then print("PackToProject: Primary path empty, trying secondary"); local dcs_secondary=listDir(secondary_path); local sec_file_s,sec_file_o=pcall(function() return File(secondary_path) end); if #dcs_secondary>0 or (sec_file_s and sec_file_o and sec_file_o.exists()) then currentPath=secondary_path;dirContents=dcs_secondary;target_path=secondary_path;builder.show("Notice","Switched to Browse: "..secondary_path) else print("PackToProject: Secondary path also empty") end end end; local displayEntries={}; local actualEntries={}; for _,item in ipairs(dirContents) do if item:match("/$") then table.insert(displayEntries,item);table.insert(actualEntries,item) end end; local dialogBuilder=AlertDialog.Builder(activity); dialogBuilder.setTitle("Select Directory to Save '"..moduleName.."': "..currentPath); if #displayEntries==0 then dialogBuilder.setMessage("No subdirectories in "..currentPath..".\nSave module directly into this folder?") else dialogBuilder.setItems(displayEntries,DialogInterface.OnClickListener{onClick=function(_,which) showDirPicker(currentPath..actualEntries[which+1]) end}) end; dialogBuilder.setPositiveButton("Save Here",DialogInterface.OnClickListener{onClick=function() builder.writeModule(moduleName,moduleCode,currentPath) end}); if currentPath~=target_path and currentPath:match("(.*/)[^/]+/$") then dialogBuilder.setNeutralButton("Go Up",DialogInterface.OnClickListener{onClick=function() local pPath=currentPath:match("(.*/)[^/]+/$"); if pPath and #pPath>=#target_path then showDirPicker(pPath) else showDirPicker(target_path) end end}) end; dialogBuilder.setNegativeButton("Cancel",nil).show() end; showDirPicker(target_path) end

function builder.updateModule(modulePath, newCode)
    local filename = modulePath:match("([^/]+)$")
    if isCriticalModule(filename) then
        showCriticalModuleDialog("edit")
        return
    end
    AlertDialog.Builder(activity).setTitle("Confirm Module Update").setMessage("You are about to overwrite the existing module:\n"..modulePath.."\n\nThis action can affect your Lua environment if the module is critical.\n\nProceed with caution?").setPositiveButton("Update Module", DialogInterface.OnClickListener{ onClick = function() local partition_remounted_rw = nil; local final_write_error_msg = ""; local perform_write = function() if builder.useRoot then local common_system_partitions = {"/system", "/vendor", "/product", "/system_ext", "/odm"}; for _, sys_path_root in ipairs(common_system_partitions) do if modulePath:sub(1, #sys_path_root + 1) == sys_path_root .. "/" then partition_remounted_rw = sys_path_root; local remount_cmd_rw = string.format("mount -o rw,remount %s", shQuote(partition_remounted_rw)); if exec(remount_cmd_rw) ~= true then local msg = "Failed to remount "..partition_remounted_rw.." as Read-Write."; final_write_error_msg=final_write_error_msg..msg.."\n"; partition_remounted_rw=nil else print(partition_remounted_rw.." remounted RW.") end; break end end end; local f,err_open=io.open(modulePath,"w"); if not f then final_write_error_msg=final_write_error_msg.."Update failed (cannot open file): "..tostring(err_open); return false end; local _,err_write=f:write(newCode); if err_write then f:close(); final_write_error_msg=final_write_error_msg.."Update failed (cannot write to file): "..tostring(err_write); return false end; local suc_close,err_close=f:close(); if not suc_close then final_write_error_msg=final_write_error_msg.."Warning (closing file): "..tostring(err_close) end; return true end; local write_successful=perform_write(); if partition_remounted_rw then local remount_cmd_ro=string.format("mount -o ro,remount %s",shQuote(partition_remounted_rw)); if exec(remount_cmd_ro) ~= true then local msg="Failed to remount "..partition_remounted_rw.." as Read-Only. Please check manually."; print(msg); final_write_error_msg=final_write_error_msg..msg.."\n" end end; if write_successful then local suc_msg="Module updated successfully:\n"..modulePath; if final_write_error_msg~="" then suc_msg=suc_msg.."\n\nNotes:\n"..final_write_error_msg end; builder.show("Update Successful",suc_msg); if builder.currentDir then builder.refreshList(builder.currentDir) end else builder.show("Update Failed",final_write_error_msg) end end }).setNegativeButton("Cancel",nil).show()
end

function builder.confirmFileDelete(fullPath, name)
    if isCriticalModule(name) then
        showCriticalModuleDialog("delete")
        return
    end
    AlertDialog.Builder(activity).setTitle("Confirm Deletion").setMessage("You are about to permanently delete the module:\n'"..name.."'\nLocation: "..fullPath.."\n\nDeleting essential modules (like 'import.lua') can severely impact your Lua environment.\n\nThis action cannot be undone. Are you absolutely sure?").setPositiveButton("Yes, Delete It", DialogInterface.OnClickListener{ onClick = function() exec("rm -rf "..shQuote(fullPath)); local file_s,file_o=pcall(function() return File(fullPath) end); if file_s and file_o and not file_o.exists() then builder.show("Deletion Successful","Module '"..name.."' has been removed.") else if builder.useRoot then builder.show("Deletion Command Sent (Root)", "Removal command for '"..name.."' sent. Please verify manually if it still exists.") else builder.show("Deletion May Have Failed","Module '"..name.."' might still be present. Please check.") end end; if builder.currentDir then builder.refreshList(builder.currentDir) end end }).setNegativeButton("Cancel",nil).setIcon(android.R.drawable.ic_dialog_alert).show()
end

builder.currentDir=nil
function builder.refreshList(path) if not path or path=="" then print("refreshList: invalid path."); path=builder.currentDir or getBasePath() or getPrimaryExternalStoragePath() end; if not path:match("/$") then path=path.."/" end; builder.currentDir=path; local entries=listDir(path); _G.listView.setAdapter(ArrayAdapter(activity,android.R.layout.simple_list_item_1, entries)); local dispPath=path; local projRoot=builder.useRoot and "/data/user/0/" or getBasePath(); if not projRoot:match("/$") then projRoot=projRoot.."/" end; local appRoot=getPrimaryExternalStoragePath(); local secAppRoot=getSecondaryExternalStoragePath(); if builder.isManagingAppModules then if builder.useRoot and path=="/" then dispPath="App Storage: FS ROOT /" elseif path:sub(1,#appRoot)==appRoot then dispPath="App Storage: ROOT/"..path:sub(#appRoot+1) elseif path:sub(1,#secAppRoot)==secAppRoot then dispPath="App Storage: ROOT(alt)/"..path:sub(#secAppRoot+1) else dispPath="App Storage: "..path end else if builder.useRoot and path=="/" then dispPath="Project Modules: FS ROOT /" elseif path:sub(1,#projRoot)==projRoot then dispPath="Project Modules: ROOT/"..path:sub(#projRoot+1) else dispPath="Project Modules: "..path end end; _G.pathLabel.setText(dispPath) end

function builder.onItemClick(_,_,position,_)
    local selName=_G.listView.getItemAtPosition(position)
    if not selName then return end
    local currDir=builder.currentDir
    if not currDir:match("/$") then currDir=currDir.."/" end
    local fullPath=currDir..selName
    if selName:match("/$") then
        builder.refreshList(fullPath)
    else 
        AlertDialog.Builder(activity).setTitle("File Options: "..selName)
        .setItems({"Edit","Delete"},DialogInterface.OnClickListener{
            onClick=function(_,which)
                if which==0 then 
                    if isCriticalModule(selName) then
                        showCriticalModuleDialog("edit")
                        return
                    end
                    local content,err=builder.readFileWithRootFallback(fullPath)
                    if not content and content~="" then 
                        builder.show("File Read Error","Could not read file:\n"..fullPath.."\n\nDetails: "..tostring(err))
                        return
                    end
                    local eBox=EditText(activity)
                    eBox.setText(content)
                    eBox.setTextSize(12)
                    eBox.setHorizontallyScrolling(true)
                    eBox.setMinLines(15)
                    eBox.setMaxLines(25)
                    eBox.setGravity(Gravity.TOP+Gravity.LEFT)
                    local scroll=HorizontalScrollView(activity)
                    local lParams=LinearLayout.LayoutParams(LinearLayout.LayoutParams.MATCH_PARENT,LinearLayout.LayoutParams.WRAP_CONTENT)
                    eBox.setLayoutParams(lParams)
                    scroll.addView(eBox)
                    AlertDialog.Builder(activity).setTitle("Editing: "..selName)
                        .setView(scroll)
                        .setPositiveButton("Save Changes",DialogInterface.OnClickListener{
                            onClick=function() builder.updateModule(fullPath,tostring(eBox.getText())) end
                        })
                        .setNegativeButton("Cancel",nil)
                        .show()
                elseif which==1 then 
                    if isCriticalModule(selName) then
                        showCriticalModuleDialog("delete")
                        return
                    end
                    builder.confirmFileDelete(fullPath,selName)
                end
            end
        })
        .show()
    end
end
local exampleModules = { { name="com.example.utils", desc="Utility functions for common tasks", code="local utils={}\nfunction utils.trim(str)\n  return str:match('^%s*(.-)%s*$')\nend\nfunction utils.clamp(val,min,max)\n  return math.max(min,math.min(max,val))\nend\nreturn utils", snippet="require \"import\"\nimport \"com.example.utils\"\nlocal u=utils\nprint(u.trim(\"  hello  \"))\nprint(u.clamp(15,0,10))" }, { name="com.example.uihelper", desc="Helper for building UI components", code="local ui={}\nfunction ui.createButton(activity,text)\n  local btn=Button(activity)\n  btn.setText(text)\n  return btn\nend\nreturn ui", snippet="require \"import\"\nimport \"com.example.uihelper\"\nlocal ui=uihelper\nlocal myButton=ui.createButton(activity,\"Click Me\")" }, { name="com.example.logger", desc="Simple logging utility", code="local logger={}\nfunction logger.log(level,msg)\n  print(string.upper(level)..\": \"..msg)\nend\nreturn logger", snippet="require \"import\"\nimport \"com.example.logger\"\nlocal log=logger\nlog.log(\"info\",\"This is an info message.\")" } }
function builder.buildExampleModule() local moduleDisplayNames={}; for i,modInfo in ipairs(exampleModules) do moduleDisplayNames[i]=modInfo.name..": "..modInfo.desc end; AlertDialog.Builder(activity).setTitle("Select Example Module").setItems(moduleDisplayNames,DialogInterface.OnClickListener{onClick=function(_,which) local selectedModule=exampleModules[which+1]; builder.writeModule(selectedModule.name,selectedModule.code); builder.show("Usage Example",selectedModule.snippet) end}).setNegativeButton("Cancel",nil).show() end
function builder.testModule() local codeToTest=_G.modulecode.getText().toString(); if codeToTest=="" then builder.show("Input Required","Please enter some Lua code in the editor to test."); return end; local loadedFunc,loadError=loadstring(codeToTest); if not loadedFunc then builder.show("Syntax Error","The code could not be compiled:\n"..loadError); return end; local origPrint=print; local capOutput={}; print=function(...)table.insert(capOutput,table.concat({...}," "))end; local sucCall,resOrErr=pcall(loadedFunc); print=origPrint; if sucCall then builder.show("Test Execution Result","Output from print() calls:\n"..(table.concat(capOutput,"\n")or"No print output recorded.")) else builder.show("Runtime Error","An error occurred during test execution:\n"..resOrErr) end end
function builder.showSettings() local options={"Use Root Permissions","About Root Access"}; AlertDialog.Builder(activity).setTitle("Settings").setMultiChoiceItems(options,{builder.useRoot,false},DialogInterface.OnMultiChoiceClickListener{onClick=function(d,w,c) if w==0 then if c then local p=io.popen("su -c id"); local idOut=p and p:read("*a"); if p then p:close() end; if idOut and idOut:match("uid=0%(") then builder.useRoot=true;builder.show("Root Enabled","Root permissions will now be used for applicable operations.") else builder.useRoot=false;builder.show("Root Access Failed","Could not verify root access or it was not granted.\nAttempted: 'su -c id'\nOutput: "..tostring(idOut)); if d and d.getListView then local lv=d.getListView(); if lv then lv.setItemChecked(0,false) end end end else builder.useRoot=false;builder.show("Root Disabled","Root permissions will not be used.") end; saveSettings() elseif w==1 then builder.show("About Root Access","Root access lets this tool manage modules across all apps and system areas. This is powerful stuff!") if d and d.getListView then local lv=d.getListView(); if lv then lv.setItemChecked(1,false) end end end end}).setPositiveButton("Done",nil).show() end
local pm_instance=nil; local function getPM() if not pm_instance then local s,p=pcall(function() return activity:getPackageManager() end); if s and p then pm_instance=p else builder.show("Error","PM not available: "..tostring(p));pm_instance=nil end end; return pm_instance end
local function getAppInfo(pkgName,flags) flags=flags or 0; local currPM=getPM(); if not currPM then return nil,"PM not available" end; local suc,appInfoOrErr=pcall(function() return currPM:getApplicationInfo(pkgName,flags) end); if suc and appInfoOrErr then return appInfoOrErr else print("Error getAppInfo "..pkgName..": "..tostring(appInfoOrErr)); return nil,tostring(appInfoOrErr) end end
function builder.runDebloater() local progDlg=ProgressDialog(activity);progDlg.setMessage("Loading application list...");progDlg.setCancelable(false);progDlg.show();builder.listAppsForDebloater(function(apps) progDlg.dismiss();if not apps then builder.show("Error","Could not load app list for Debloater.");return end;if #apps==0 then builder.show("Information","No applications found to list.");return end;local appDispList={};for _,appD in ipairs(apps)do table.insert(appDispList,string.format("%s\n(%s) - %s",appD.label,appD.packageName,appD.type))end;AlertDialog.Builder(activity).setTitle("Debloater - Select Application").setItems(appDispList,DialogInterface.OnClickListener{onClick=function(_,idx)builder.showDebloaterAppOptions(apps[idx+1])end}).setNegativeButton("Cancel",nil).show()end)end
function builder.listAppsForDebloater(callback) local currPM=getPM();if not currPM then if callback then callback(nil)end;return end;local apps={};local instApps_s,instApps_o=pcall(function()return currPM:getInstalledApplications(PackageManager.GET_META_DATA)end);if not instApps_s or not instApps_o then if callback then callback(nil)end;return end;local appListJ=instApps_o;local appListL={};if appListJ and appListJ.size then for i=0,appListJ:size()-1 do table.insert(appListL,appListJ:get(i))end else appListL=luajava.astable(appListJ)or{}end;activity.runOnUiThread(function()for _,appInf in ipairs(appListL)do local appLbl_s,appLbl_o=pcall(function()return currPM:getApplicationLabel(appInf).toString()end);local appLbl=(appLbl_s and appLbl_o)or appInf.packageName;local pkgN=appInf.packageName;local appT="User";local isSys=false;local isUpdSys=false;if appInf.flags then if(luajava.bitwiseAnd(appInf.flags,ApplicationInfo.FLAG_SYSTEM)~=0)then appT="System";isSys=true end;if(luajava.bitwiseAnd(appInf.flags,ApplicationInfo.FLAG_UPDATED_SYSTEM_APP)~=0)then appT="Updated System";isUpdSys=true end end;local lIntent=luajava.newInstance("android.content.Intent",Intent.ACTION_MAIN,nil);lIntent:addCategory(Intent.CATEGORY_LAUNCHER);lIntent:setPackage(pkgN);local resInf_s,resInf_o=pcall(function()return currPM:queryIntentActivities(lIntent,0)end);if resInf_s and resInf_o and resInf_o.size and resInf_o:size()>0 then appT=appT.." (Launcher)"end;table.insert(apps,{label=appLbl,packageName=pkgN,type=appT,isSystem=isSys,isUpdatedSystem=isUpdSys,appInfo=appInf})end;table.sort(apps,function(a,b)return string.lower(a.label)<string.lower(b.label)end);if callback then callback(apps)end end)end
function builder.showDebloaterAppOptions(appD) local opts={"Disable for current user (Recommended)","Attempt full uninstall (DANGEROUS)"};AlertDialog.Builder(activity).setTitle(appD.label.."\n("..appD.packageName..")").setItems(opts,DialogInterface.OnClickListener{onClick=function(_,optIdx)if optIdx==0 then builder.debloatForCurrentUser(appD)elseif optIdx==1 then builder.debloatFromDeviceWarning(appD)end end}).setNegativeButton("Cancel",nil).show()end
function builder.debloatForCurrentUser(appD) local cmd=string.format("pm uninstall --user 0 %s",shQuote(appD.packageName));builder.show("Action: Disable App","Attempting to disable for current user (0):\n"..cmd.."\n\nThis typically hides the app for user 0. For system apps, it may uninstall updates and revert to factory version or disable it if full removal isn't possible without deeper system mods.");local res=exec(cmd);builder.show("Result","Disable command for '"..appD.label.."' sent.\nResult code: "..tostring(res))end
function builder.debloatFromDeviceWarning(appD) AlertDialog.Builder(activity).setTitle("EXTREME DANGER: Full Uninstall").setMessage(string.format("You are attempting to FULLY UNINSTALL:\n'%s'\nPackage: %s\n\nWARNING: If this is a system app, removing its files can make your device UNSTABLE or UNBOOTABLE. This action is hard to undo and might require a system reflash.\n\nAre you absolutely certain?",appD.label,appD.packageName)).setIcon(android.R.drawable.ic_dialog_alert).setPositiveButton("Yes, Proceed with Full Uninstall",DialogInterface.OnClickListener{onClick=function()builder.debloatFromDeviceExecute(appD)end}).setNegativeButton("NO! CANCEL!",nil).show()end
function builder.debloatFromDeviceExecute(appD) builder.show("Processing Full Removal","Attempting to forcefully remove "..appD.label..". This may take a moment.");exec(string.format("pm uninstall --user 0 %s",shQuote(appD.packageName)));local freshAppInf,_=getAppInfo(appD.packageName,PackageManager.GET_META_DATA);local srcDir=freshAppInf and freshAppInf.sourceDir;local datDir=freshAppInf and freshAppInf.dataDir;if not srcDir then builder.show("Error","Cannot find source directory for "..appD.packageName.." post-disable. Cannot proceed with 'rm -rf'.");return end;local cmds={};local remnt=nil;local sysRts={"/system","/vendor","/product","/system_ext","/odm"};for _,r in ipairs(sysRts)do if srcDir:sub(1,#r)==r then remnt=r;table.insert(cmds,string.format("mount -o rw,remount %s",shQuote(r)));break end end;local apkPth=srcDir;if srcDir:match("%.apk$")then local p=srcDir:match("(.*/)[^/]+%.apk$");if p then apkPth=p end end;table.insert(cmds,string.format("rm -rf %s",shQuote(apkPth)));if datDir and datDir~=""then table.insert(cmds,string.format("rm -rf %s",shQuote(datDir)))end;table.insert(cmds,string.format("rm -rf /data/dalvik-cache/*/%s*",shQuote(appD.packageName)));if remnt then table.insert(cmds,string.format("mount -o ro,remount %s",shQuote(remnt)))end;local fullCmd=table.concat(cmds," && ");local res=exec(fullCmd);builder.show((res==true and "Removal Successful (Attempted)" or "Removal Command Sent"),appD.label..(res==true and " appears to have been forcefully removed." or " removal command chain executed with result: "..tostring(res)).."\nA REBOOT IS STRONGLY RECOMMENDED to ensure changes take full effect.")end
function builder.runInitialDebloaterWarning() AlertDialog.Builder(activity).setTitle("DANGER - SYSTEM DEBLOATER").setMessage("You are about to access the System App Debloater.\n\nThis powerful tool can remove ANY application, including those essential for your device's operation.\n\nIncorrect usage can lead to an UNSTABLE or UNBOOTABLE system. There is NO UNDO feature.\n\nDo you understand these significant risks and wish to proceed?").setPositiveButton("Yes, I Understand & Accept Risks",DialogInterface.OnClickListener{onClick=function()builder.runDebloater()end}).setNegativeButton("NO, CANCEL",nil).setIcon(android.R.drawable.ic_dialog_alert).show()end
function builder.updateToolboxVisibility() if _G.btnToolbox then _G.btnToolbox.setVisibility(builder.useRoot and View.VISIBLE or View.GONE) end end
function builder.openToolbox() local toolbox_items={"Debloater (System App Remover)"}; AlertDialog.Builder(activity).setTitle("Root Toolbox").setItems(toolbox_items,DialogInterface.OnClickListener{onClick=function(_,idx)if toolbox_items[idx+1]:match("Debloater")then builder.runInitialDebloaterWarning()end end}).setNegativeButton("Cancel",nil).show()end

_G.layout = {
  LinearLayout, orientation="vertical", layout_width="fill_parent", layout_height="fill_parent", padding="16dp",
  { EditText, id="modulename", visibility="visible", hint="Module name e.g. com.example.myutils", layout_width="fill_parent", layout_height="wrap_content" },
  { ScrollView, id="modulecode_scrollview", visibility="visible", layout_width="fill_parent", layout_height="0dp", layout_weight="1",
    { EditText, id="modulecode", layout_width="fill_parent", layout_height="wrap_content", hint="-- Your Lua module code here", singleLine=false, minLines=10, gravity="top|left", inputType="textMultiLine|textNoSuggestions", textIsSelectable=true }},
  { LinearLayout, id="main_buttons_panel1", visibility="visible", orientation="horizontal", layout_width="fill_parent", layout_height="wrap_content",
    { Button, text="Build Module", layout_weight="1", onClick=function() local mn,mc=_G.modulename.getText().toString(),_G.modulecode.getText().toString();if mn==""or mc==""then builder.show("Input Required","Please enter both a module name and some code to build.")else builder.writeModule(mn,mc)end end},
    { Button, text="Pack to Project", layout_weight="1", onClick=function() local mn,mc=_G.modulename.getText().toString(),_G.modulecode.getText().toString();if mn==""or mc==""then builder.show("Input Required","Please enter module name and code to pack.")else builder.writeModuleToProject(mn,mc)end end}},
  { LinearLayout, id="main_buttons_panel2", visibility="visible", orientation="horizontal", layout_width="fill_parent", layout_height="wrap_content",
    { Button, text="Example Module", layout_weight="1", onClick=builder.buildExampleModule },
    { Button, id="btnInstall", text="Install from Storage", layout_weight="1" }, 
    { Button, text="Test Module", layout_weight="1", onClick=builder.testModule }
  },
  { LinearLayout, id="main_buttons_panel3", visibility="visible", orientation="horizontal", layout_width="fill_parent", layout_height="wrap_content",
    { Button, id="btnManage", text="Manage Modules", layout_weight="1" },
    { Button, id="btnToolbox", text="Toolbox", layout_weight="1", visibility="gone"}, 
    { Button, id="btnSettings", text="Settings", layout_weight="1" }
  },
  { TextView, id="pathLabel", text="", visibility="gone", layout_width="fill_parent", layout_height="wrap_content", textSize="12sp", padding="4dp" },
  { ListView, id="listView", visibility="gone", layout_width="fill_parent", layout_height="0dp", layout_weight="1" },
  { LinearLayout, id="btnRow", visibility="gone", orientation="horizontal", layout_width="fill_parent", layout_height="wrap_content",
    { Button, id="btnToggleProjectApp", text="Manage Project Modules and Code", layout_weight="1.2"},
    { Button, id="btnUp", text="Go Up", layout_weight="0.8" },
    { Button, id="btnClose", text="Close Manager", layout_weight="1" }}
}

activity.setContentView(loadlayout(_G.layout))
loadSettings() 
if _G.btnToolbox then builder.updateToolboxVisibility() end 

if _G.btnManage then _G.btnManage.onClick=function()builder.isManagingAppModules=false;if _G.btnToggleProjectApp then _G.btnToggleProjectApp.setText("Manage Project Modules and Code")end;local sPath=builder.useRoot and"/data/user/0/"or getBasePath();if not sPath:match("/$")then sPath=sPath.."/"end;if _G.modulename then _G.modulename.setVisibility(View.GONE)end;if _G.modulecode_scrollview then _G.modulecode_scrollview.setVisibility(View.GONE)end;if _G.main_buttons_panel1 then _G.main_buttons_panel1.setVisibility(View.GONE)end;if _G.main_buttons_panel2 then _G.main_buttons_panel2.setVisibility(View.GONE)end;if _G.main_buttons_panel3 then _G.main_buttons_panel3.setVisibility(View.GONE)end;if _G.pathLabel then _G.pathLabel.setVisibility(View.VISIBLE)end;if _G.listView then _G.listView.setVisibility(View.VISIBLE)end;if _G.btnRow then _G.btnRow.setVisibility(View.VISIBLE)end;builder.refreshList(sPath)end end
if _G.btnSettings then _G.btnSettings.onClick=builder.showSettings end
if _G.btnToolbox then _G.btnToolbox.onClick=builder.openToolbox end
if _G.btnInstall then _G.btnInstall.onClick = builder.installModuleFromStorage end 

if _G.listView then _G.listView.setOnItemClickListener(AdapterView.OnItemClickListener{onItemClick=function(p,v,pos,i)builder.onItemClick(p,v,pos,i)end}) end
if _G.btnToggleProjectApp then _G.btnToggleProjectApp.onClick=function()local pList;if builder.isManagingAppModules then builder.isManagingAppModules=false;_G.btnToggleProjectApp.setText("Manage Project Modules and Code");pList=builder.useRoot and"/data/user/0/"or getBasePath()else builder.isManagingAppModules=true;_G.btnToggleProjectApp.setText("Manage App Modules");pList=getPrimaryExternalStoragePath();local iList=listDir(pList);if #iList==0 then local sP=getSecondaryExternalStoragePath();if sP~=pList then print("ToggleApp: Primary empty, try secondary "..sP);local sfs,sfo=pcall(function()return File(sP)end);local sList=listDir(sP);if #sList>0 or(sfs and sfo and sfo.exists())then pList=sP;builder.show("Notice","Browse: "..sP)else print("ToggleApp: Secondary also empty");builder.show("Warning","Cannot access external paths.")end end end end;if pList then if not pList:match("/$")then pList=pList.."/"end;builder.refreshList(pList)else builder.show("Error","Cannot determine path.")end end end
if _G.btnUp then _G.btnUp.onClick=function()local rootPath;if builder.isManagingAppModules then rootPath=builder.useRoot and"/"or getPrimaryExternalStoragePath();if builder.useRoot and builder.currentDir=="/"then builder.show("Top Level","FS Root (/)");return end;if not builder.useRoot and builder.currentDir and builder.currentDir:sub(1,#getSecondaryExternalStoragePath())==getSecondaryExternalStoragePath()then rootPath=getSecondaryExternalStoragePath()end else rootPath=builder.useRoot and"/"or getBasePath();if builder.useRoot and builder.currentDir=="/"then builder.show("Top Level","FS Root (/)");return end end;if not rootPath:match("/$")then rootPath=rootPath.."/"end;if builder.currentDir==rootPath then local msg=builder.isManagingAppModules and(builder.useRoot and"FS Root (/)"or"App Storage Root")or(builder.useRoot and"FS Root (/)"or"Project Modules Root");builder.show("Top Level",msg);return end;local parentDir=builder.currentDir:match("(.*/)[^/]+/$");if parentDir then if builder.useRoot and parentDir=="/"then builder.refreshList(parentDir)elseif #parentDir>=#rootPath then builder.refreshList(parentDir)else builder.refreshList(rootPath)end else builder.refreshList(rootPath);local msgNav=builder.isManagingAppModules and(builder.useRoot and"FS Root (/)"or"App Storage Root")or(builder.useRoot and"FS Root (/)"or"Project Modules Root");builder.show("Top Level",msgNav.." (Nav to root)")end end end
if _G.btnClose then _G.btnClose.onClick=function()if _G.modulename then _G.modulename.setVisibility(View.VISIBLE)end;if _G.modulecode_scrollview then _G.modulecode_scrollview.setVisibility(View.VISIBLE)end;if _G.main_buttons_panel1 then _G.main_buttons_panel1.setVisibility(View.VISIBLE)end;if _G.main_buttons_panel2 then _G.main_buttons_panel2.setVisibility(View.VISIBLE)end;if _G.main_buttons_panel3 then _G.main_buttons_panel3.setVisibility(View.VISIBLE)end;if _G.pathLabel then _G.pathLabel.setVisibility(View.GONE)end;if _G.listView then _G.listView.setVisibility(View.GONE)end;if _G.btnRow then _G.btnRow.setVisibility(View.GONE)end;builder.currentDir=nil;if _G.btnToolbox then builder.updateToolboxVisibility()end end end

local bp=getBasePath(); local fsbp,fobp=pcall(function()return File(bp)end)
if not(fsbp and fobp and fobp.exists())then print("Warning: Base path "..tostring(bp).." does not exist post-load.")end

return builder