function integritycheck.startcheck()
  local function exec(cmd)
    local f = io.popen(cmd)
    if f then
      local out = f:read("*a")
      f:close()
      return out:match("^%s*(.-)%s*$") or "unknown"
    end
    return "unknown"
  end

  local results = {}

  print("=== Starting Full Integrity Check ===\n")

  -- Bootloader Check
  local bootloader = exec("getprop ro.boot.verifiedbootstate")
  print("Bootloader State: " .. bootloader)
  if bootloader == "orange" then
    table.insert(results, "FAILED: Bootloader is in ORANGE state (unlocked).")
  else
    table.insert(results, "PASSED: Verified boot is OK.")
  end

  -- Flash Lock Check
  local flashlock = exec("getprop ro.boot.flash.locked")
  print("Flash Lock: " .. flashlock)
  if flashlock == "0" then
    table.insert(results, "FAILED: Flash lock is DISABLED.")
  else
    table.insert(results, "PASSED: Flash lock is ENABLED.")
  end

  -- Verified Boot Mode
  local vboot = exec("getprop ro.boot.veritymode")
  print("Verified Boot Mode: " .. vboot)
  if vboot ~= "enforcing" then
    table.insert(results, "FAILED: Verified boot NOT enforcing.")
  else
    table.insert(results, "PASSED: Verified boot is enforcing.")
  end

  -- SELinux
  local selinux = exec("getenforce")
  print("SELinux Mode: " .. selinux)
  if selinux ~= "Enforcing" then
    table.insert(results, "FAILED: SELinux is NOT enforcing.")
  else
    table.insert(results, "PASSED: SELinux is Enforcing.")
  end

  -- Security Patch Check
  local patch = exec("getprop ro.build.version.security_patch")
  print("Security Patch: " .. patch)
  local year, month = patch:match("(%d+)%-(%d+)")
  year = tonumber(year or 0)
  month = tonumber(month or 0)
  if year < 2023 or (year == 2023 and month < 12) then
    table.insert(results, "FAILED: Security patch too OLD (< Dec 2023).")
  else
    table.insert(results, "PASSED: Security patch is recent.")
  end

  -- Final Result
  print("\n--- CHECK RESULTS ---")
  local failed = false
  for i,v in ipairs(results) do
    print(v)
    if v:find("FAILED") then
      failed = true
    end
  end

  print("\n--- INTEGRITY CHECK DONE ---")
  if failed then
    print("STATUS: INTEGRITY COMPROMISED!")
  else
    print("STATUS: DEVICE PASSES ALL INTEGRITY TESTS.")
  end
end